# Procedure to redshift an HI moment map a galaxy at redshift z1 mapped
# with a cell size of dx1 
#                        For VLA upgrade simulation. JEH 1/31/99
#
procedure HIz(mom0,mom2,snr)
string mom0="mom0"      {prompt="MOM0 input (mJy/b km/s)"}
string mom2="mom2"      {prompt="MOM2 input (m/s rms)"}
string snr="snr"        {prompt="output S/N map"}
real z1=0.005		{prompt="Redshift of input image"}
real dx1=5.0		{prompt="pixel size of input image (arcsec)"}
real FWHM1=18.0	        {prompt="FWHM resolution of input image (arcsec)"}
real z2=0.10		{prompt="Redshift of output image"}
real dx2=0.5		{prompt="pixel size of output image (arcsec)"}
real FWHM2=1.6	        {prompt="FWHM resolution of output image at z=0 (arcsec)"}
real Ho=75		{prompt="Hubbles Constant"}
real qo=0.1		{prompt="deceleration parameter"}
real Tsys=28		{prompt="Tsys (K)"}
real eta=0.55		{prompt="efficiency"}
int  Nant=27		{prompt="Number of Array Elements"}
int  Nif=2		{prompt="Number of IF pairs"}
real texp=2700.0        {prompt="exposure time in hours"}

begin
	real bin_factor,rebin, fade_factor, snr_factor
        real Dlum1,Dlum2,kpc1,kpc2,Srms,smooth,temp

        Dlum1=(qo*z1+(qo-1)*(sqrt(1+2*qo*z1)-1))*3E5/Ho/qo/qo
        Dlum2=(qo*z2+(qo-1)*(sqrt(1+2*qo*z2)-1))*3E5/Ho/qo/qo
        kpc1=dx1*Dlum1/206.265/(1+z1)/(1+z1)
        kpc2=dx2*Dlum2/206.265/(1+z2)/(1+z2)
	print("Redshifting ",mom0," from z=",z1," to z=",z2,":")
	printf("  Lum. Distance: was %9.5e Mpc, shift to %9.5e Mpc \n",Dlum1,Dlum2)
	printf("  pixel scale:   was %7.4f kpc/pix, shift to %7.4f kpc/pix \n",kpc1,kpc2)

# Srms is the rms for a 1 kHz wide channel:
        Srms=0.12*Tsys/eta/sqrt(Nant*(Nant-1)*Nif*texp*0.001)
	printf("  RMS for 1 kHz channel %9.5e mJy/b \n",Srms)

	fade_factor=(Dlum1*dx1*FWHM2/Dlum2/dx2/FWHM1)**2 * (1+z2)**3 / (1+z1)

        snr_factor = fade_factor*1.419/sqrt(1+z2)/Srms
        #bin_factor=(dx1/dx2)*(Dlum1/Dlum2)*((1+z2)/(1+z1))**2
        bin_factor=kpc1/kpc2
        rebin=kpc2/kpc1
        smooth=FWHM2*(1+z2)/dx2/2.35
	printf("  Fade by %9.5e, rebin %5.2f x %5.2f \n",fade_factor,rebin,rebin)

# First, get mom2 into km/s
        imarith(mom2,"/",1000,snr)

# I pad mom2 with some minimum dispersion, like 6 km/s
        imreplace(snr,value=6.0,upper=6,lower=indef)
        imfunct(snr,snr,"sqrt")
	imarith(mom0,"/",snr,snr,divzero=0)
	magnify(snr,snr,bin_factor,bin_factor,fluxc+,logfile-)
	imarith(snr,"*",snr_factor,snr)
	gaus(snr,snr,smooth,nsigma=4)

# Edit the headers to indicate the new pixel size and S/N units
        temp=FWHM2*(1+z2)
	printf("  Final image has resolution of %5.2f arcsec and a cellsize of %5.2f arcsec\n",temp,dx2)
        hedit(snr,"BUNIT","S/N",ver-,show-,up+)
        temp=-dx2/3600
        hedit(snr,"CDELT1",temp,ver-,show-,up+)
        hedit(snr,"CD1_1",temp,ver-,show-,up+)
        hedit(snr,"CD1_1",temp,ver-,show-,up+)
        temp=dx2/3600
        hedit(snr,"CDELT2",temp,ver-,show-,up+)
        hedit(snr,"CD2_2",temp,ver-,show-,up+)
#
end
#
#  magnify does not properly conserve flux; need to calculate correction factor (from redshift.cl):
# 
# struct *list
#	real bin_factor, rebin, fade_factor, corr_factor
#	real mean, temp, sum1, sum2
#	int Ibin,Npix
#	   del("tempfile")
#	   imstat(in,field="npix,mean",lower=indef,upper=indef,format=no,>>"tempfile")
#	   list="tempfile"
#          k=fscan(list,Npix,mean)
#          sum1=Npix*mean
#          del("tempfile")
#	   magnify(in,out,bin_factor,bin_factor,fluxc+,logfile-)
#	   imstat(out,field="npix,mean",lower=indef,upper=indef,format=no,>>"tempfile")
#	   list="tempfile"
#	   k=fscan(list,Npix,mean)
#	   sum2=Npix*mean
#	   del("tempfile")
#	   corr_factor=sum1/sum2
#	   imarith(out,"*",corr_factor,out)
#	   printf("  Magnifying by %6.2f and using correction factor %7.5f \n",rebin,corr_factor)
