# Procedure to redshift an HI moment map a galaxy at redshift z1 mapped
# with a cell size of dx1. Like HIz, but adds noise to final
# image
#                        For VLA upgrade simulation. JEH 1/31/99
#
procedure HIz2(mom0,mom2,snr)
string mom0="mom0"      {prompt="MOM0 input (mJy/b km/s)"}
string mom2="mom2"      {prompt="MOM2 input (m/s rms)"}
string snr="snr"        {prompt="output S/N map + noise"}
int outsize=200         {prompt="output image size (pixels)"}
real z1=0.005		{prompt="Redshift of input image"}
real dx1=5.0		{prompt="pixel size of input image (arcsec)"}
real FWHM1=18.0	        {prompt="FWHM resolution of input image (arcsec)"}
real z2=0.10		{prompt="Redshift of output image"}
real dx2=0.5		{prompt="pixel size of output image (arcsec)"}
real FWHM2=1.6	        {prompt="FWHM resolution of output image at z=0 (arcsec)"}
real Ho=75		{prompt="Hubbles Constant"}
real qo=0.1		{prompt="deceleration parameter"}
real Tsys=28		{prompt="Tsys (K)"}
real eta=0.55		{prompt="efficiency"}
int  Nant=27		{prompt="Number of Array Elements"}
int  Nif=2		{prompt="Number of IF pairs"}
real texp=2700.0        {prompt="exposure time in hours"}

begin
	real bin_factor,rebin, fade_factor, snr_factor
        real Dlum1,Dlum2,kpc1,kpc2,Srms,smooth,temp,noise
        int x1,x2,y1,y2,size

        Dlum1=(qo*z1+(qo-1)*(sqrt(1+2*qo*z1)-1))*3E5/Ho/qo/qo
        Dlum2=(qo*z2+(qo-1)*(sqrt(1+2*qo*z2)-1))*3E5/Ho/qo/qo
        kpc1=dx1*Dlum1/206.265/(1+z1)/(1+z1)
        kpc2=dx2*Dlum2/206.265/(1+z2)/(1+z2)
	print("Redshifting ",mom0," from z=",z1," to z=",z2,":")
	printf("  Lum. Distance: was %9.5e Mpc, shift to %9.5e Mpc \n",Dlum1,Dlum2)
	printf("  pixel scale:   was %7.4f kpc/pix, shift to %7.4f kpc/pix \n",kpc1,kpc2)

# Srms is the rms for a 1 kHz wide channel:
        Srms=0.12*Tsys/eta/sqrt(Nant*(Nant-1)*Nif*texp*0.001)
	printf("  RMS for 1 kHz channel %9.5e mJy/b \n",Srms)

	fade_factor=(Dlum1*dx1*FWHM2/Dlum2/dx2/FWHM1)**2 * (1+z2)**3 / (1+z1)

        snr_factor = fade_factor*1.419/sqrt(1+z2)/Srms
        #bin_factor=(dx1/dx2)*(Dlum1/Dlum2)*((1+z2)/(1+z1))**2
        bin_factor=kpc1/kpc2
        rebin=kpc2/kpc1
	printf("  Fade by %9.5e, rebin %5.2f x %5.2f \n",fade_factor,rebin,rebin)

# First, get mom2 into km/s
        imdel("tempz")
        imarith(mom2,"/",1000,"tempz")

# I pad mom2 with some minimum dispersion, like 6 km/s
        imreplace("tempz",value=6.0,upper=6,lower=indef)
        imfunct("tempz","tempz","sqrt")
	imarith(mom0,"/","tempz","tempz",divzero=0)
	magnify("tempz","tempz",bin_factor,bin_factor,fluxc+,logfile-)
	imarith("tempz","*",snr_factor,"tempz")
#
# Copy into output image then add noise 
# requires packages noao,artdata be loaded for mknoise
	mknoise(snr,ncol=outsize,nline=outsize,background=0,poisson-,ncosray=0,rdnoise=0,comments-)
	imgets("tempz","i_naxis1")
	size=int(imgets.val)
        x1=int((outsize-size)/2+1)
        x2=x1+size-1
	imgets("tempz","i_naxis2")
	size=int(imgets.val)
        y1=int((outsize-size)/2+1)
        y2=y1+size-1
        imcopy("tempz",snr//"["//x1//":"//x2//","//y1//":"//y2//"]")
        smooth=FWHM2*(1+z2)/dx2/2.35
# THIS DOES NOT WORK AS I THINK IT SHOULD - S/N OF SMOOTHED IMAGE DOES
# NOT = (S/N)RAW/FWHM**2.  What I need to do is calculate sigma of
# the smoothed image, then divide by this factor
# The 1.39 factor is a fudge factor; gives s/n~1 for z=0.3. 
        noise=(smooth*1.39)**4
        # printf("smooth= %9.5f     background= %9.5f \n",smooth,noise)
	mknoise(snr,out=snr,background=noise,poisson+,ncosray=0,rdnoise=0)
        mkheader(snr,"tempz",append-)
        imarith(snr,"-",noise,snr)
	gaus(snr,snr,smooth,nsigma=4)

# Edit the headers to indicate the new pixel size and S/N units
        temp=FWHM2*(1+z2)
	printf("  Final image has resolution of %5.2f arcsec and a cellsize of %5.2f arcsec\n",temp,dx2)
        temp=-dx2/3600
        hedit(snr,"CDELT1",temp,ver-,show-,up+)
        hedit(snr,"CD1_1",temp,ver-,show-,up+)
        hedit(snr,"CD1_1",temp,ver-,show-,up+)
        temp=dx2/3600
        hedit(snr,"CDELT2",temp,ver-,show-,up+)
        hedit(snr,"CD2_2",temp,ver-,show-,up+)
        hedit(snr,"BUNIT","S/N",ver-,show-,up+)
end



